# Author: Yue Ma  Date: November, 2010

########################################################################
# Given a polynomial f and a vector of monomials, this routine returns an 
# operator 
#
# Inputs:
#        f     ---  a polynomial
#        monos ---  a vector of monomials
#        vars  ---  a list of variables
#
# Outputs:
#        Oper  ---  a vector of linear conditions
#        b     ---  a vector of numbers       
########################################################################

Operator_f:=proc(f,monos::Vector,vars)
  local m,G,M,p,i,b,Oper,h;

  m:=Dimension(monos);  
  
  G:=Matrix(m,m,symbol=a): 
  
  h:=expand(LinearAlgebra[Transpose](monos).G.monos-f); 
  
  M:=[coeffs(h,vars)];  

  p:=nops(M);
  b:=Vector(p);
  Oper:=Vector(p);     
  for i from 1 to p do
      if ldegree(M[i])=0 then
          b[i]:=-tcoeff(M[i]);
          Oper[i]:=M[i]+b[i];
      else
          b[i]:=0;
          Oper[i]:=M[i];
      fi:
  od:

  return Oper,b;
end proc:


###################################################################

 sub_A:=proc(A::Vector)
     local i,j,subA,n,p,B,yy,num;
           p:=Dimension(A);
           subA:=[];
           num:=Vector(p);
           for i from 1 to p do
              B:=[op(A[i])]; 
              if type(B[1],integer)=true then
                 subA:=[op(subA),convert(B,Vector[row])]; 
                 num[i]:=1;
              else
                 n:=nops(B);  
                 num[i]:=n;
                 yy:=Matrix(n,2);
                 for j from 1 to n do
                    yy[j,1]:=op(B[j])[1];
                    yy[j,2]:=op(B[j])[2];
                 od:
                 
                 subA:=[op(subA),yy];
              end if:
           od:    
     return subA,num;  #list
  end proc:



#################################################################
# This routine computes a basis of the multivariate polynomial. 
# Input:
#      d --- degree
#      n --- the number of variables
# Output:
#      S --- vector referenced by integer in canonical ordering
#################################################################

vecnumb:=proc(d,n)  
  local i,S,v,N;
    S:=[];
    N:=(d+n)!/(d!*n!); 
      for i from 0 to N-1 do
        v:=combinat[inttovec](i,n); 
        S:=[op(S),v];
      end do;
      RETURN(S);
end proc:


############################################################## 
# This routine converts a integer vector to monomial. 
# Input:
#      v --- integer vector 
#      L --- variable vector
# Output:
#      s --- monomial
##############################################################
vecmo:=proc(v,L)  
 local i,n,s;
   n:=nops(L);
   s:=1;
   for i from 1 to n do
     s:=s*L[i]^(v[i]);
   end do;
   RETURN(s);
end proc:


###############################################################
# This routine returns a vector of monomials with dimension n 
# and degree under d.
# Input:
#        n     ---  a integer 
#        d     ---  a integer
#        vars  ---  a list of variables
# Output:
#        L     ---  a vector of monomials
#################################################################

get_monos:=proc(n,d,vars::list)
  local vars_n,m,L_1,L_2,i,L;

    vars_n:=nops(vars);
    m:=binomial(vars_n+d,d);

    if m<n then
       error("The input number d needs to be larger.");
    end if:

    L_1:=vecnumb(d,nops(vars));
    L_2:=Vector(m);
    for i from 1 to nops(L_1) do
          L_2[i]:=vecmo(L_1[i],vars);
    end do:
    L:=L_2[1..n]:

    return L;

end proc:
